import java.awt.*;
import java.awt.geom.*;
import java.lang.Math;
import java.text.NumberFormat;

/** Circulo en 2D. Subclase de Forma.*/

public class Circulo extends Forma
{
/** El centro del Circulo.*/
	private Point2D.Double centro;
/** El radio del Circulo.*/
	private double radio;

/** Constructor. Instancia un nuevo Circulo.
*@param c Centro del circulo. Punto entero.
*@param r Radio del circulo. Real doble, valor absoluto.*/

	Circulo (Point c, double r)
	{
		centro = new Point2D.Double (c.getX(), c.getY());
		radio = Math.abs(r);
	}

/** Escala la forma respecto al punto central en un factor dado.
* @param c Punto respecto del cual se hace el escalamiento.
* @param factor Factor usado en el escalamiento. >1 corresponde a amplificacion, <1 corresponde a reduccion.*/

	public void escalar (Point c, double factor)
	{
		double x = centro.getX();
		double y = centro.getY();
		x = c.getX()+ factor*(x - c.getX());
		y = c.getY()+ factor*(y - c.getY());
		radio = radio*factor;
		centro.setLocation(x, y);
	}

/** Rota la forma respecto al punto central en un angulo dado.
* @param c Punto respecto del cual se hace la rotacion.
* @param deltaAngulo Angulo de rotacion. En radianes.*/
	
	public void rotar (Point c, double deltaAngulo)
	{
		double centerX = centro.getX() - c.getX();
		double centerY = centro.getY() - c.getY();
		double distancia = Math.sqrt (centerX*centerX + centerY*centerY);
		double anguloInicial = Math.atan2 (centerY, centerX);
		centerX = distancia*Math.cos (anguloInicial + deltaAngulo) + c.getX();
		centerY = distancia*Math.sin (anguloInicial + deltaAngulo) + c.getY();
		centro.setLocation(centerX, centerY);
	}	
	
/** Traslada la forma, desplazandola en sentido X e Y.
* @param deltaX Desplazamiento en sentido horizontal.
* @param deltaY Desplazamiento en sentido vertical.*/

	public void trasladar (double deltaX, double deltaY)
	{
		double x = centro.getX();
		double y = centro.getY();
		centro.setLocation(x + deltaX, y + deltaY);
	}

/** Retorna un string que representa al circulo. Los dobles son formateados a un maximo de tres decimales.
*@return String descriptivo. "Circulo: Centro; radio".*/
	
	public String toString ()
	{
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(3);
		return ("Circulo: Centro = (" + nf.format(centro.getX()) + ", " + nf.format(centro.getY()) + "); Radio = " + nf.format(radio) + ".");
	}

/** Verifica si este circulo es igual a otro objeto dado.
*@param otherObject Otro objeto a comparar.
*@return Verdadero si son iguales, falso de lo contrario.*/

	public boolean equals (Object otherObject)
	{
		if (this == otherObject)
			return true;
		if (otherObject == null)
			return false;
		if (getClass() != otherObject.getClass())
			return false;
		Circulo other = (Circulo)otherObject;
		return (centro.equals(other.centro) && radio == other.radio);
	}

/** Clona este circulo.
*@return Un objecto identico a este circulo.*/

	public Object clone ()
	{
		Point c = new Point ((int)centro.getX(), (int)centro.getY());
		Object clon = new Circulo (c, radio);
		return clon;
	}
}