import java.util.ArrayList;
import java.awt.Point;

/** Arreglo de formas en 2D. Subclase de Forma.*/

public class Figura extends Forma
{
/** Arreglo de formas.*/
	private ArrayList<Forma> dibujo;
	
/** Constructor. Instancia una nueva figura.*/
	Figura ()
	{
		dibujo = new ArrayList<Forma>();
	}

/** Escala la forma respecto al punto central en un factor dado.
* @param centro Punto respecto del cual se hace el escalamiento.
* @param factor Factor usado en el escalamiento. >1 corresponde a amplificacion, <1 corresponde a reduccion.*/

	public void escalar(Point centro, double factor)
	{
		for (int i = 0; i < dibujo.size(); i++)
			dibujo.get(i).escalar (centro, factor);
	}

/** Rota la forma respecto al punto central en un angulo dado.
* @param centro Punto respecto del cual se hace la rotacion.
* @param deltaAngulo Angulo de rotacion. En radianes.*/

	public void rotar(Point centro, double deltaAngulo)
	{
		for (int i = 0; i < dibujo.size(); i++)
			dibujo.get(i).rotar (centro, deltaAngulo);
	}

/** Traslada la forma, desplazandola en sentido X e Y.
* @param deltaX Desplazamiento en sentido horizontal.
* @param deltaY Desplazamiento en sentido vertical.*/

	public void trasladar(double deltaX, double deltaY)
	{
		for (int i = 0; i < dibujo.size(); i++)
			dibujo.get(i).trasladar (deltaX, deltaY);
	}

/** Agrega un circulo a la figura.
*@param c Centro del circulo. Punto entero.
*@param r Radio del circulo. Real doble.*/

	public void addCirculo (Point c, double r)
	{
		dibujo.add (new Circulo (c, r));
	}

/** Agrega un rectangulo a la figura.
*@param a Esquina del rectangulo. Punto entero.
*@param b Esquina del rectangulo. Punto entero.*/
	
	public void addRectangulo (Point a, Point b)
	{
		dibujo.add (new Rectangulo (a, b));
	}

/** Retorna un string que representa a la figura. Se presenta la cantidad de formas incluidas.
*@return String descriptivo. "Figura. Numero de formas incluidas.".*/

	public String toString ()
	{
		return ("Figura. Contiene " + dibujo.size() + " formas.");
	}

/** Imprime en pantalla una descripcion de la figura. Se presenta una descripcion de la figura y de cada forma incluida.*/
	
	public void printFigura ()
	{
		System.out.print(toString());
		if (dibujo.size() > 0)
			System.out.print(" A continuacion se detalla cada una:\n");
		else
			System.out.print("\n");
		for (int i = 0; i < dibujo.size(); i++)
			System.out.println(i + ": " + dibujo.get(i).toString());
	}
}